﻿
-- SilageBunkerHUD
-- Shows Silage Amount in the Silo Bunkers

-- @author  	Syrius
-- @version 	v 1.0
-- @date  		27.12.2014

print("SilageBunkerHUD v 1.0 by Syrius");

SilageBunkerHUD = {
	config = g_modsDirectory.."/SB_HUD_config.xml",
	edit = false,
	PosX = 0.826,
	PosY = 0.198,
	width = 0.178,
	height = 0.15,
	stepY = 0.001,
	stepX = 0.001,
	show = true,
	show_recycler_info = true,
	overlayPath = g_currentModDirectory.."Background.dds",
	moveIconPath = g_currentModDirectory.."move.dds",
	silo = {},
	recycler_height = 0.025,
	bunker = {}
};

function SilageBunkerHUD:loadMap(name)
	-- load config
	SilageBunkerHUD:loadConfig(self.config);

	self.Overlay=createImageOverlay(self.overlayPath);
	self.moveIcon=createImageOverlay(self.moveIconPath);
	if g_currentMission.onCreateLoadedObjects ~= nil then
		a = 1;
		for k, v in pairs(g_currentMission.onCreateLoadedObjects) do
			if v.stationName == "BunkerSilo" then
				self.silo[a] = g_currentMission.onCreateLoadedObjects[v.id].nodeId;
				a = a + 1;
			end;
			if v.bunkerUseTime then
				self.bunker = v;
			end;
		end;
	end;
	if self.show_recycler_info then
		self.recycler_height = 0;
	end;
end;
function SilageBunkerHUD:deleteMap()
end;
function SilageBunkerHUD:mouseEvent(posX, posY, isDown, isUp, button)
end;
function SilageBunkerHUD:keyEvent(unicode, sym, modifier, isDown)
end;
function SilageBunkerHUD:update(dt)
	if InputBinding.hasEvent(InputBinding.TOGGLE_SBunkerHUD) then
		self.show=not self.show;
	end;
	if InputBinding.hasEvent(InputBinding.EDIT_SBunkerHUD) then
		self.edit = not self.edit;
--		print("--- SBHud Edit Mode ".. (tostring(self.edit)) .."! ---");
		if not self.edit then
			-- save config
			SilageBunkerHUD:saveConfig(self.config);
		end;
	end;
	if g_currentMission.showHelpText and not self.show then
		g_currentMission:addHelpButtonText(g_i18n:getText("TOGGLE_SBunkerHUD"), InputBinding.TOGGLE_SBunkerHUD);
	end;
	if self.edit and Input.keyPressedState[Input.KEY_lshift] then
		if Input.keyPressedState[Input.KEY_left] then
			self.PosX = self.PosX - self.stepX;
		end;
		if Input.keyPressedState[Input.KEY_right] then
			self.PosX = self.PosX + self.stepX;
		end;
		if Input.keyPressedState[Input.KEY_up] then
			self.PosY = self.PosY + self.stepY;
		end;
		if Input.keyPressedState[Input.KEY_down] then
			self.PosY = self.PosY - self.stepY;
		end;
	end;
end;
function SilageBunkerHUD:updateTick(dt)
end;
function SilageBunkerHUD:draw()
	if self.PosX < 1 and self.PosY < 1 and self.PosX > 0 and self.PosY > 0 then
		if self.show == true then
			local Font=0.015;
			local TPosX=self.PosX + 0.007;
			local TPosYBegin=self.PosY + 0.09;
			local APosX=self.PosX+0.14;
			local startPosY = 1;
			local height = self.height;
			
			setTextBold(false);
			
			height = self.height - self.recycler_height;
			renderOverlay(self.Overlay, self.PosX, self.PosY - 0.02 + self.recycler_height, self.width, height);
			if self.edit then
				renderOverlay(self.moveIcon, self.PosX - 0.008, self.PosY + 0.114, pxToNormal(32, "x"), pxToNormal(32, "y"));
			end;
			setTextColor(1, 1, 1, 1);
			renderText(TPosX + 0.005, TPosYBegin + Font, Font, g_i18n:getText("SB_HUD_SILO"));
			renderText(APosX - 0.02, TPosYBegin + Font, Font, g_i18n:getText("SB_HUD_AMOUNT"));
			
			setTextAlignment(RenderText.ALIGN_RIGHT);
			renderText(TPosX + 0.08, TPosYBegin + Font, Font, g_i18n:getText("SB_HUD_CF"));
			
			setTextAlignment(RenderText.ALIGN_LEFT);
			setTextColor(1, 1, 0, 1);
			renderText(TPosX, TPosYBegin-Font, Font, g_i18n:getText("SB_HUD_SILO1")..":");
			renderText(TPosX, TPosYBegin-Font* 2, Font, g_i18n:getText("SB_HUD_SILO2")..":");
			renderText(TPosX, TPosYBegin-Font* 3, Font, g_i18n:getText("SB_HUD_SILO3")..":");
			renderText(TPosX, TPosYBegin-Font* 4, Font, g_i18n:getText("SB_HUD_SILO4")..":");
			renderText(TPosX, TPosYBegin-Font* 5, Font, g_i18n:getText("SB_HUD_SILO5")..":");
			
			if self.show_recycler_info then
				setTextColor(1, 1, 1, 1);
				renderText(TPosX, TPosYBegin-Font* 6 - 0.005, Font, g_i18n:getText("SB_HUD_RECYCLER") .. ":");
				setTextColor(1, 1, 0, 1);
			end;
			
			setTextAlignment(RenderText.ALIGN_RIGHT);
			for key, value in pairs(self.silo) do
				obj = g_currentMission:getNodeObject(value);
				if obj.state == 0 and obj.fillLevel > 0 then
					setTextColor(0, 1, 0, 1);
				elseif obj.state == 2 then
					setTextColor(0.3, 0.1, 0, 1);
				elseif obj.state == 0 and obj.fillLevel == 0 then
					setTextColor(1, 1, 1, 1);
				else
					setTextColor(0, 1, 1, 1);
				end;
				compact = round( tostring(obj.compactedFillLevel) / tostring(obj.fillLevel) * 100);
				ferment = round(obj.fermentingTime / obj.fermentingDuration * 100);
				if obj.state == 1 then
					renderText(TPosX + 0.08, TPosYBegin-Font*startPosY, Font, ferment .. "%"); -- fermentation
				end;
				if obj.state == 0 and obj.compact ~= 100 then
					renderText(TPosX + 0.08, TPosYBegin-Font*startPosY, Font, compact .. "%"); -- compacting
				end;
				renderText(APosX, TPosYBegin-Font*startPosY, Font, comma_value(round(obj.fillLevel)) .. " " .. g_i18n:getText("fluid_unit_short"));
				renderText(APosX + 0.03, TPosYBegin-Font*startPosY, Font, "(" .. round(obj.fillLevel / obj.capacity * 100) .. "%)");
				startPosY = startPosY + 1;
			end;
			if self.show_recycler_info then
				setTextColor(1, 1, 1, 1);
				renderText(APosX, TPosYBegin-Font* 6 - 0.005, Font, comma_value(round(self.bunker.bunkerFillLevel)) .. " " .. g_i18n:getText("fluid_unit_short"));
			end;
		end;
	end;
end;

addModEventListener(SilageBunkerHUD);


function round(num, idp)
	if Utils.getNoNil(num, 0) > 0 then
		local mult = 10^(idp or 0);
		-- res = math.floor(num * mult + 0.5) / mult;
		res = math.floor(num * mult) / mult;
	else 
		return 0;
	end;
	if res < 1 then res = 1 end;
	return res;
end;

function comma_value(n) -- credit http://richard.warburton.it
	local left,num,right = string.match(n,'^([^%d]*%d)(%d*)(.-)$')
	return left..(num:reverse():gsub('(%d%d%d)','%1,'):reverse())..right
end

function SilageBunkerHUD:loadConfig(file)
	local hudXml;
	if fileExists(file) then -- load config
		hudXml = loadXMLFile("SB_HUD_XML", file, "options");
	else -- create config file
		hudXml = createXMLFile("SB_HUD_XML", file, "options");
		setXMLFloat(hudXml, "options.HUD#X", self.PosX);
		setXMLFloat(hudXml, "options.HUD#Y", self.PosY);
		-- setXMLFloat(hudXml, "options.HUD#width", self.width);
		-- setXMLFloat(hudXml, "options.HUD#height", self.height);
		setXMLBool(hudXml, "options.show", self.show);
		setXMLBool(hudXml, "options.recycler#show", self.show_recycler_info);
		saveXMLFile(hudXml);
	end;
	if hudXml ~= nil then
		self.PosX = getXMLFloat(hudXml, "options.HUD#X");
		self.PosY = getXMLFloat(hudXml, "options.HUD#Y");
		-- self.width = getXMLFloat(hudXml, "options.HUD#width");
		-- self.height = getXMLFloat(hudXml, "options.HUD#height");
		self.show = getXMLBool(hudXml, "options.show");
		self.show_recycler_info = getXMLBool(hudXml, "options.recycler#show");
	end;
end;

function SilageBunkerHUD:saveConfig(file)
	local hudXml;
	if not fileExists(file) then
		hudXml = createXMLFile("SB_HUD_XML", file, "options");
	else
		hudXml = loadXMLFile("SB_HUD_XML", file, "options");
	end;
	if hudXml ~= nil then
		setXMLFloat(hudXml, "options.HUD#X", self.PosX);
		setXMLFloat(hudXml, "options.HUD#Y", self.PosY);
		-- setXMLFloat(hudXml, "options.HUD#width", self.width);
		-- setXMLFloat(hudXml, "options.HUD#height", self.height);
		setXMLBool(hudXml, "options.show", self.show);
		setXMLBool(hudXml, "options.recycler#show", self.show_recycler_info);
		saveXMLFile(hudXml);
	end;
end;

function print_r (t, indent, done)
  done = done or {}
  indent = indent or ''
  local nextIndent -- Storage for next indentation value
  for key, value in pairs (t) do
    if type (value) == "table" and not done [value] then
      nextIndent = nextIndent or
          (indent .. string.rep(' ',string.len(tostring (key))+2))
          -- Shortcut conditional allocation
      done [value] = true
      print (indent .. "[" .. tostring (key) .. "] => Table {");
      print  (nextIndent .. "{");
      print_r (value, nextIndent .. string.rep(' ',2), done)
      print  (nextIndent .. "}");
    else
      print  (indent .. "[" .. tostring (key) .. "] => " .. tostring (value).."")
    end
  end
end

local targetAspectRatio = 16/9; -- = 1920/1080;
local aspectRatioRatio = g_screenAspectRatio / targetAspectRatio;
local sizeRatio = 1;
if g_screenWidth > 1920 then
	sizeRatio = 1920 / g_screenWidth;
end;

-- px are in targetSize for 1920x1080
function pxToNormal(px, dimension, fullPixel)
	local ret;
	if dimension == 'x' then
		ret = (px / 1920) * sizeRatio;
	else
		ret = (px / 1080) * sizeRatio * aspectRatioRatio;
	end;
	if fullPixel == nil or fullPixel then
		ret = getFullPx(ret, dimension);
	end;

	return ret;
end;

function getFullPx(n, dimension)
	if dimension == 'x' then
		return round(n * g_screenWidth) / g_screenWidth;
	else
		return round(n * g_screenHeight) / g_screenHeight;
	end;
end;

function getPxToNormalConstant(widthPx, heightPx)
	return widthPx/g_screenWidth, heightPx/g_screenHeight;
end;
 